/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <string>
#include <sstream>
#include <stdlib.h>
#include <iostream>

#include "AxisAlignedBoundingBox.hh"
#include "XmlElement.hh"

AxisAlignedBoundingBox::AxisAlignedBoundingBox ( point_t parMin, point_t parMax )
{
    myMinPoint = parMin;
    myMaxPoint = parMax;
}

AxisAlignedBoundingBox::AxisAlignedBoundingBox(XmlElement* parXml)
{
    if(!parXml)
        throw;

    std::string theValue;
    size_t theStart;
    size_t theEnd;

    //Min
    theValue = parXml->getAttributeValue("min", "0 0 0");
    theStart = 0;
    theEnd = 0;

    theEnd = theValue.find_first_of(' ', theStart);
    myMinPoint.x = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.find_first_of(' ', theStart);
    myMinPoint.z = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    myMinPoint.y = atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    //Max
    theValue = parXml->getAttributeValue("max", "0 0 0");
    theStart = 0;
    theEnd = 0;

    theEnd = theValue.find_first_of(' ', theStart);
    myMaxPoint.x = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.find_first_of(' ', theStart);
    myMaxPoint.z = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    myMaxPoint.y = atof(theValue.substr(theStart, theEnd-theStart).c_str());

}


XmlElement* AxisAlignedBoundingBox::toXml()
{
    std::stringstream   theStream;
    XmlElement*         theXml;
    
    theXml = new XmlElement("AABBOX");
    
    theStream << myMinPoint.x <<" " << myMinPoint.z << " " << myMinPoint.y;
    theXml->addAttribute("min", theStream.str());
    
    theStream.str("");
    theStream << myMaxPoint.x <<" " << myMaxPoint.z << " " << myMaxPoint.y;
    theXml->addAttribute("max", theStream.str());

    return theXml;
}

