/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <stdlib.h>
#include <string.h>
#include <iostream>
#include "FileHandler.hh"


FileHandler::FileHandler()
{
    myDataSize = 0;
    myFp = NULL;
}


FileHandler::~FileHandler()
{
    std::vector<std::string*>::const_iterator cii;
    for(cii = myStrings.begin(); cii != myStrings.end(); cii++)
    {
        delete *cii;
    }
}

uint32_t FileHandler::swapBytes(uint32_t parValue) {
    uint32_t theResult = 0;
    uint8_t* tmp = (uint8_t*)(&parValue);

    theResult |= (uint32_t)(*tmp  << 24  & 0xff000000); tmp++;
    theResult |= (uint32_t)((*tmp << 16) & 0x00ff0000); tmp++;
    theResult |= (uint32_t)((*tmp <<  8) & 0x0000ff00); tmp++;
    theResult |= (uint32_t)((*tmp      ) & 0x000000ff);

    return theResult;
}


size_t FileHandler::read(void *ptr, size_t size, size_t nmemb)
{
    size_t rval;
    rval = fread(ptr, size, nmemb, myFp);
    if(!myEndian)
    {
        unsigned int i;
        uint32_t* d = (uint32_t*)ptr;
        for(i = 0; i < (size*nmemb)/sizeof(uint32_t); i++) {
            d[i] = swapBytes(d[i]);
        }
    }
    return rval;
}

size_t FileHandler::write(const void* ptr, size_t size, size_t nmemb)
{
    if(!myEndian)
    {
        size_t theRval;
        void* theData = malloc(nmemb*size);
        memcpy(theData, ptr, nmemb*size);
        unsigned int i;
        uint32_t* d = (uint32_t*)theData;
        for(i = 0; i < (size*nmemb)/sizeof(uint32_t); i++) {
            d[i] = swapBytes(d[i]);
        }
        theRval = fwrite(theData, size, nmemb, myFp);
        free(theData);
        return theRval;
    }
    else
    {
        return fwrite(ptr, size, nmemb, myFp);
    }
}


uint32_t FileHandler::addString ( std::string parString )
{
    uint32_t theOffset = 0;
    uint32_t theHash = hashString(parString);

    std::map<int,int>::iterator it;
    it = myStringOffsets.find(theHash);
    if(it != myStringOffsets.end())
    {
        return myDataOffset + ((*it).second);
    }

    theOffset = myDataSize;
    myStringOffsets[theHash] = myDataSize;
    myDataSize += parString.length()+1;
    myStrings.push_back(new std::string(parString));
    return myDataOffset+theOffset;
}


std::string FileHandler::getString(size_t pos)
{
    char data[500];
    std::string rVal;
    
    size_t start = ftell(myFp);
    fseek(myFp, pos-1, SEEK_SET);

    if(fgetc(myFp) == 0) {
        fgets(data, 500, myFp);
        fseek(myFp, start, SEEK_SET);
        rVal.append(data);
        return rVal;
    }
    fseek(myFp, start, SEEK_SET);
    rVal.append("Not following NULL");
    return rVal;
}

void FileHandler::writeData()
{
    std::vector<std::string*>::const_iterator cii;
    for(cii = myStrings.begin(); cii != myStrings.end(); cii++)
    {
        fwrite((*cii)->c_str(), (*cii)->length()+1, 1, myFp);
    }
}

size_t FileHandler::tell()
{
    return ftell(myFp);
}

uint32_t FileHandler::hashString(std::string parString)
{
    uint32_t hashVal = 0;
    size_t theLength = parString.length();
    if(theLength == 0)
        return 0;

    for (size_t i = 0; i < (theLength-1); i++)
    {
        hashVal += (parString.at(i)*31)^(theLength-(1+i));
        hashVal ^= (hashVal << 5) + (hashVal >> 2) + parString.at(i);
    }
    hashVal += parString.at(theLength-1);

    return hashVal;
}

