/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef FILEHANDLER_HH
#define FILEHANDLER_HH

#include <string>
#include <vector>
#include <map>
#include <stdio.h>
#include <stdint.h>

typedef int Endian;

class FileHandler
{
    private:
        Endian myEndian;
        FILE*  myFp;
        size_t myDataOffset;
        size_t myDataSize;
        std::vector<std::string*> myStrings;
        std::map<int, int> myStringOffsets;
    
    public:
        FileHandler();
        ~FileHandler();
        uint32_t    swapBytes(uint32_t parValue);
        size_t      read(void *ptr, size_t size, size_t nmemb);
        size_t      write(const void *ptr, size_t size, size_t nmemb);
        std::string getString(size_t pos);
        uint32_t    addString(std::string parString);
        size_t      tell();
        uint32_t hashString(std::string parStr);
        
        void        endian(Endian parEndian)
        {
            myEndian = parEndian;
        }
        Endian endian()
        {
            return myEndian;
        }

        void fp(FILE* parFp)
        {
            myFp = parFp;
        }
        
        void length(size_t parSize)
        {
            myDataOffset = parSize;
        }
        
        void writeData();

};

#endif // FILEHANDLER_HH
