/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "FlatSection.hh"
#include <stdio.h>
#include <iostream>
#include <string>
#include <sstream>
#include "util.h"
#include "sgb.h"
#include "sections.h"
#include "Partition.hh"
#include "FileHandler.hh"
#include "XmlElement.hh"



void FlatSection::readSection(FileHandler* parFh, block_t* theSection)
{
    myPartition = new Partition(parFh, 0);
}
        
void FlatSection::readSection(XmlElement* parXml)
{
    if(!parXml)
        return;
    myPartition = new Partition(parXml->getChildNode("PARTITION_ID"));
}


FlatSection::FlatSection()
{
    myPartition = NULL;
}

FlatSection::~FlatSection()
{
    if(myPartition)
        delete myPartition;
}

std::string FlatSection::toXml()
{
    std::stringstream   theStream;
    std::string         theRval;
    
    theStream << "\t<FLAT_SECTION>" << std::endl;
    if(myPartition)
        theStream << myPartition->toXml();
    theStream << "\t</FLAT_SECTION>" << std::endl;
    
    theRval = theStream.str();
    return theRval;
}

size_t FlatSection::rawLength()
{
    size_t theRval = sizeof(section_head);
    if(myPartition)
        theRval += myPartition->rawLength();
    return theRval;
}

void FlatSection::toRaw(FileHandler* parFh)
{
    section_head theHead;
    theHead.name = parFh->swapBytes(SECTION_FLAT);
    theHead.length = rawLength();
    theHead.blocks = 1;
    parFh->write(&theHead, sizeof(section_head), 1);
    if(myPartition)
        myPartition->toRaw(parFh);
}
