/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <string>
#include <sstream>
#include <stdlib.h>

#include "Matrice.hh"
#include "sgb.h"
#include "util.h"
#include "FileHandler.hh"
#include "XmlElement.hh"

Matrice::Matrice ( FILE* parFp, int parEndian, int parId )
{
    matrix_t theMatrix;
    freadEndian(&theMatrix, sizeof(matrix_t), 1, parFp, parEndian);
    myId      = parId;
    myOffsetX = theMatrix.offset_x;
    myOffsetY = theMatrix.offset_y;
    myOffsetZ = theMatrix.offset_z;
    
    myOrientationX = theMatrix.orientation_x;
    myOrientationY = theMatrix.orientation_y;
    myOrientationZ = theMatrix.orientation_z;
    
    myOther1    = theMatrix.a;
    myOther2    = theMatrix.b;
    
    myParentId = theMatrix.parent;

}

Matrice::Matrice(XmlElement* parXml)
{
    if(!parXml)
        throw;

    myId        = atoi(parXml->getAttributeValue("id", "-1").c_str());
    myParentId  = atoi(parXml->getAttributeValue("ParentId", "-1").c_str());

    std::string theValue;
    size_t theStart;
    size_t theEnd;

    //Offset
    theStart = 0;
    theEnd = 0;
    theValue = parXml->getAttributeValue("Offset", "0 0 0");
    
    theEnd = theValue.find_first_of(' ', theStart);
    myOffsetX = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.find_first_of(' ', theStart);
    myOffsetZ = atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    myOffsetY = atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    
    //Orientation
    theStart = 0;
    theEnd = 0;
    theValue = parXml->getAttributeValue("Orientation", "0 0 0");
    
    theEnd = theValue.find_first_of(' ', theStart);
    myOrientationX = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.find_first_of(' ', theStart);
    myOrientationZ= atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    myOrientationY = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    //Unknown
    theStart = 0;
    theEnd = 0;
    theValue = parXml->getAttributeValue("unknown", "0 0");
    
    theEnd = theValue.find_first_of(' ', theStart);
    myOther1 = atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    myOther2 = atof(theValue.substr(theStart, theEnd-theStart).c_str());
}

std::string Matrice::toXml(int parLevel)
{
    std::stringstream   theStream;
    std::string         theRval;
    std::string         theBaseIndent = "\t\t\t\t";
    
    if(parLevel == 2)
        theBaseIndent += "\t";

    theStream << theBaseIndent << "<MATRIX";
    if(myId != -1)
        theStream << " id=\"" << myId << "\"";
    theStream << " Offset=\""<< myOffsetX << " " << myOffsetZ << " " << myOffsetY << "\"";
    theStream << " Orientation=\"" << myOrientationX << " " << myOrientationZ << " " << myOrientationY << "\"";
    theStream << " unknown=\"" << myOther1<< " " << myOther2<< "\"";
    
    if(myParentId != -1)
        theStream << " ParentId=\""<<myParentId<<"\"";
    
    theStream << " />" << std::endl;
    theRval = theStream.str();
    return theRval;
}

void Matrice::toRaw ( FileHandler* parFh )
{
    matrix_t theMatrix;
    theMatrix.offset_x = myOffsetX;
    theMatrix.offset_y = myOffsetY;
    theMatrix.offset_z = myOffsetZ;
    
    theMatrix.orientation_x = myOrientationX;
    theMatrix.orientation_y = myOrientationY;
    theMatrix.orientation_z = myOrientationZ;
    
    theMatrix.a = myOther1;
    theMatrix.b = myOther2;
    theMatrix.parent = myParentId;
    parFh->write(&theMatrix, sizeof(matrix_t), 1);

}
