/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "NodeSection.hh"
#include "ObjectId.hh"
#include "FileHandler.hh"
#include "sections.h"
#include "XmlElement.hh"


NodeSection::NodeSection(std::string parTagName, SECTIONS parTag):
    myTagName("NODE_SECTION")
{
    myTagName = parTagName;
    myTag = parTag;
}


NodeSection::~NodeSection()
{
    std::vector<ObjectId*>::const_iterator cii;
    for(cii = myObjectIds.begin(); cii != myObjectIds.end(); cii++)
    {
        delete (*cii);
    }
}


void NodeSection::readSection(FILE* parFp, int endian, int num)
{
    for(int i = 0; i < num; i++)
    {
        ObjectId* theObjectId = new ObjectId(parFp, endian);
        myObjectIds.push_back(theObjectId);
    }
}
        

void NodeSection::readSection(XmlElement* parXml)
{
    if(!parXml)
        return;

    int theNum = parXml->nChildNode(std::string("OBJ_ID"));
    for(int i = 0; i < theNum; i++)
    {
        ObjectId* theObjectId = new ObjectId(parXml->getChildNode(std::string("OBJ_ID"), i));
        myObjectIds.push_back(theObjectId);
    }
}


std::string NodeSection::toXml()
{
    std::string         theRval;
    std::stringstream   theStream;


    theStream << "\t<"<<myTagName<<">"<<std::endl;

    std::vector<ObjectId*>::const_iterator cii;
    for(cii = myObjectIds.begin(); cii != myObjectIds.end(); cii++)
    {
        theStream << (*cii)->toXml();
    }

    theStream << "\t</"<< myTagName <<">"<<std::endl;
    theRval = theStream.str();
    return theRval;
}


size_t NodeSection::rawLength()
{
    size_t theSize = sizeof(section_head);
    std::vector<ObjectId*>::const_iterator cii;
    for(cii = myObjectIds.begin(); cii != myObjectIds.end(); cii++)
    {
        theSize += (*cii)->rawLength();
    }
    return theSize;
}
        

void NodeSection::toRaw(FileHandler* parFh)
{
    section_head theHead;
    theHead.name = parFh->swapBytes(myTag);
    theHead.blocks = myObjectIds.size();
    theHead.length = rawLength();

    parFh->write(&theHead, sizeof(section_head), 1);
    std::vector<ObjectId*>::const_iterator cii;
    for(cii = myObjectIds.begin(); cii != myObjectIds.end(); cii++)
    {
        (*cii)->toRaw(parFh);
    }
}

