/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <vector>
#include <string>
#include <sstream>

#include "OcclSection.hh"
#include "Occluder.hh"
#include "FileHandler.hh"
#include "XmlElement.hh"
#include "sections.h"


OcclSection::~OcclSection()
{
    std::vector<Occluder*>::const_iterator cii;
    for(cii = myOccluders.begin(); cii != myOccluders.end(); cii++)
    {
        delete (*cii);
    }

}


void OcclSection::readSection(FILE* parFp, int endian, int num)
{
    for(int i = 0; i < num; i++)
    {
        Occluder* theOccluder = new Occluder(parFp, endian);
        myOccluders.push_back(theOccluder);
    }
}


void OcclSection::readSection(XmlElement* parXml)
{
    if(!parXml)
        return;

    int theNum = parXml->nChildNode(std::string("OCCL_OBJ"));
    for(int i = 0; i < theNum; i++)
    {
        Occluder* theOccluder = new Occluder(parXml->getChildNode(std::string("OCCL_OBJ"), i));
        myOccluders.push_back(theOccluder);
    }
}


XmlElement* OcclSection::toXml()
{
    XmlElement* theElement;
    theElement = new XmlElement("OCCL_SECTION");

    std::vector<Occluder*>::const_iterator cii;
    for(cii = myOccluders.begin(); cii != myOccluders.end(); cii++)
    {
        theElement->appendChild((*cii)->toXml());
    }

    return theElement;
}


void OcclSection::toRaw(FileHandler* parFh)
{
    section_head theHead;
    theHead.name = parFh->swapBytes(SECTION_OCCL);
    theHead.length = rawLength();
    theHead.blocks = myOccluders.size();

    parFh->write(&theHead, sizeof(section_head), 1);
    std::vector<Occluder*>::const_iterator cii;
    for(cii = myOccluders.begin(); cii != myOccluders.end(); cii++)
    {
        (*cii)->toRaw(parFh);
    }
}


size_t OcclSection::rawLength()
{
    return sizeof(section_head)+(sizeof(occl_t)*myOccluders.size());
}

