/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <string>
#include <sstream>
#include <stdlib.h>

#include "Occluder.hh"
#include "util.h"
#include "FileHandler.hh"
#include "XmlElement.hh"
        

Occluder::Occluder(FILE* parFp, int parEndian)
{
    occl_t theData;
    freadEndian(&theData, sizeof(occl_t), 1, parFp, parEndian);
    myName      = getString(parFp,theData.str1, NULL);
    myResource  = getString(parFp,theData.str2, NULL);
    myPoints[0] = theData.p1;
    myPoints[1] = theData.p2;
    myPoints[2] = theData.p3;
    myPoints[3] = theData.p4;
}
        
Occluder::Occluder(XmlElement* parXml)
{
    if(!parXml)
        throw;

    myName      = parXml->getAttributeValue(std::string("name"));
    myResource  = parXml->getAttributeValue(std::string("file"));

    int numPoints = parXml->nChildNode(std::string("point"));
    if(numPoints != 4)
        throw;

    for(int i = 0; i < numPoints; i++)
    {
        int theId;
        std::string thePos;
        XmlElement* theXmlPoint = parXml->getChildNode(std::string("point"), i);
        theId = atoi(theXmlPoint->getAttributeValue(std::string("id"), "0").c_str()); //FIXME don't default all to 0
        thePos = theXmlPoint->getAttributeValue(std::string("pos"), "0 0 0");

        size_t theStart = 0;
        size_t theEnd   = thePos.find_first_of(' ', theStart);
        myPoints[theId].x = atof(thePos.substr(theStart, (theEnd - theStart)).c_str());
        
        theStart = thePos.find_first_not_of(' ', theEnd); 
        theEnd   = thePos.find_first_of(' ', theStart);
        myPoints[theId].z = atof(thePos.substr(theStart, (theEnd - theStart)).c_str());
        
        theStart = thePos.find_first_not_of(' ', theEnd); 
        theEnd   = thePos.find_first_of(' ', theStart);
        myPoints[theId].y = atof(thePos.substr(theStart, (theEnd - theStart)).c_str());
    }
}


/*
void Occluder::setName(std::string parName)
{
    myName = parName;
}

void Occluder setResource(std::string parResource)
{
    myResource = parResource;
}
*/

int Occluder::lengthRaw()
{
    return (2*4)+(4*sizeof(point_t));
}

XmlElement* Occluder::toXml()
{
    XmlElement* theElement;
    std::string         theRval;
    //Build XML

    theElement = new XmlElement("OCCL_OBJ");
    theElement->addAttribute("name", myName);
    theElement->addAttribute("file", myResource);

    for(int i = 0; i < 4; i++)
    {
        std::stringstream   theStream;
        XmlElement* thePos = new XmlElement("point");
        
        theStream << i;
        thePos->addAttribute("id", theStream.str());

        theStream.str("");
        theStream << myPoints[0].x<<" "<<myPoints[0].z<<" "<<myPoints[0].y;
        thePos->addAttribute("pos", theStream.str());
        theElement->appendChild(thePos);
    }

    return theElement;
}


void Occluder::toRaw(FileHandler* parFh)
{
    occl_t theData;
    theData.str1 = parFh->addString(myName);
    theData.str2 = parFh->addString(myResource);
    theData.p1  = myPoints[0];
    theData.p2  = myPoints[1];
    theData.p3  = myPoints[2];
    theData.p4  = myPoints[3];
    parFh->write(&theData, sizeof(occl_t), 1);
}
