/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <string>
#include <sstream>
#include <stdlib.h>

#include "PartitionChild.hh"
#include "Sphere.hh"
#include "util.h"
#include "FileHandler.hh"
#include "XmlElement.hh"

PartitionChild::PartitionChild(FileHandler* parFh)
{
    flatdata_t data;
    parFh->read(&data, sizeof(flatdata_t), 1);

    mySphere    = new Sphere(data.s2);
    myPoint1    = new Point(data.p1, 0);
    myPoint2    = new Point(data.p2, 1);
    myId        = data.id;
}


PartitionChild::PartitionChild(XmlElement* parXml)
{
    myPoint1 = NULL;
    myPoint2 = NULL;
    mySphere = NULL;
    if(!parXml)
        throw;
    mySphere    = new Sphere(parXml->getChildNode("SPHERE"));
    myPoint1    = new Point(parXml->findChildwithAttribute("POINT", "id", "0"));
    myPoint2    = new Point(parXml->findChildwithAttribute("POINT", "id", "1"));
    myId        = atoi(parXml->getAttributeValue("id", "0").c_str());
}


PartitionChild::~PartitionChild()
{
    delete mySphere;
    delete myPoint1;
    delete myPoint2;
}

std::string PartitionChild::toXml()
{
    std::stringstream   theStream;
    std::string         theRval;
    theStream << "\t\t\t<CHILD_OBJ id=\"" << myId << "\">" << std::endl;
    if(mySphere)
        theStream << mySphere->toXml();
    if(myPoint1)
        theStream << myPoint1->toXml();
    if(myPoint2)
        theStream << myPoint2->toXml();
    theStream << "\t\t\t</CHILD_OBJ>" << std::endl;
    
    theRval = theStream.str();
    return theRval;
}

size_t PartitionChild::rawLength()
{
    return sizeof(flatdata_t);
}

void PartitionChild::toRaw(FileHandler* parFh)
{
    flatdata_t theData;
    theData.id = myId;
    theData.p1 = myPoint1->point();
    theData.p2 = myPoint2->point();
    theData.s1.center_x = 0;
    theData.s1.center_y = 0;
    theData.s1.center_z = 0;
    theData.s1.radius   = 0;
    theData.s2 = mySphere->sphere();
    theData.zero = 0;
    parFh->write(&theData, sizeof(flatdata_t), 1);
}
