/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <string>
#include <sstream>
#include <stdlib.h>

#include "Point.hh"
#include "XmlElement.hh"

Point::Point ( point_t parPoint, int parId)
{
    myId    = parId;
    myPoint = parPoint;
}


Point::Point(XmlElement* parXml)
{
    if(!parXml)
        throw;

    myId    = atoi(parXml->getAttributeValue("id", "0").c_str());
    
    std::string theValue = parXml->getAttributeValue("position", "0 0 0");
    size_t theStart = 0;
    size_t theEnd = 0;

    theEnd = theValue.find_first_of(' ', theStart);
    myPoint.x = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.find_first_of(' ', theStart);
    myPoint.z = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    myPoint.y = atof(theValue.substr(theStart, theEnd-theStart).c_str());

}


std::string Point::toXml()
{
    std::stringstream   theStream;
    std::string         theRval;
    
    theStream << "\t\t\t\t<POINT id=\"" << myId << "\" position=\"" << myPoint.x << " " << myPoint.z << " " << myPoint.y << "\" />" << std::endl;
    
    theRval = theStream.str();
    return theRval;
}
