/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <iostream>
#include <string>
#include <sstream>
#include <stdlib.h>

#include "Sphere.hh"
#include "XmlElement.hh"

Sphere::Sphere(sphere_t parData)
{
    mySphere = parData;
}

Sphere::Sphere(XmlElement* parXml)
{
    if(!parXml)
        throw;
    
    std::string theValue;
    size_t theStart;
    size_t theEnd;

    //Centre
    theStart = 0;
    theEnd = 0;
    theValue = parXml->getAttributeValue("Centre", "0 0 0");
    
    theEnd = theValue.find_first_of(' ', theStart);
    mySphere.center_x = atof(theValue.substr(theStart, theEnd-theStart).c_str());

    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.find_first_of(' ', theStart);
    mySphere.center_z = atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    theStart = theValue.find_first_not_of(' ', theEnd);
    theEnd = theValue.length();
    mySphere.center_y = atof(theValue.substr(theStart, theEnd-theStart).c_str());
    
    
    //Radius
    mySphere.radius = atof(parXml->getAttributeValue("Radius", "0").c_str());
}


std::string Sphere::toXml(int parLevel)
{
    std::stringstream   theStream;
    std::string         theRval;
    
    std::string theBaseIndent ="\t\t\t\t";
    if(parLevel == 2)
        theBaseIndent += "\t";
    
    theStream << theBaseIndent << "<SPHERE Centre=\"";
    theStream << mySphere.center_x;
    theStream << " ";
    theStream << mySphere.center_z;
    theStream << " ";
    theStream << mySphere.center_y;
    theStream << "\" Radius=\"";
    theStream << mySphere.radius;
    theStream << "\" />"<<std::endl;
    theRval = theStream.str();
    return theRval;
}

