/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef XMLELEMENT__HH
#define XMLELEMENT__HH

#include <string>

class XmlElement
{
    private:
        std::string myName;
        std::string myValue;
        XmlElement* myParent;
        XmlElement* myFirstAttr;
        XmlElement* myFirstChild;
        XmlElement* myPrevSibling;
        XmlElement* myNextSibling;


    public:
        enum NodeType {
            TYPE_ELEMENT,
            TYPE_ATTRIBUTE
        };
        XmlElement(std::string parName, std::string parValue = "", NodeType parType = XmlElement::TYPE_ELEMENT);
        ~XmlElement();
        static XmlElement* parseString(std::string parData);
        std::string toXml(bool parIndent = false, int parLevel = 0);
        void addAttribute(std::string parName, std::string parValue);
        void appendChild(XmlElement* parChild);
        XmlElement* getAttribute(std::string parName);
        std::string getAttributeValue(std::string parName, std::string parDefault = "NULL");
        XmlElement* getChildNode(std::string, int parNum = 0);
        unsigned int nChildNode(std::string parName);
        XmlElement* parent() { return myParent; }
        XmlElement* findChildwithAttribute(std::string parName, std::string parAttName, std::string parAttValue);

    private:
        NodeType    myType;
        std::string indent(int parLevel);

        class XmlTookenizer
        {
            private:
                std::string myData;
                char* myDatPosition;
                int myPosition;
                int findNonWhiteSpace();
                int findEndString();
            public:
                enum XmlTookenType {
                    TOKEN_TEXT,
                    TOKEN_QUOTEDTEXT,
                    TOKEN_TAGSTART,
                    TOKEN_TAGEND,
                    TOKEN_TAGCLOSE,
                    TOKEN_EQUAL,
                    TOKEN_EMPTYELEM,
                    TOKEN_ERROR
                };
                typedef struct {
                    XmlTookenType myType;
                    std::string   myData;
                } XmlTooken; 

                XmlTookenizer(std::string);
                XmlTooken* getToken(XmlTooken* parPrev = NULL);
        };
        XmlElement(XmlTookenizer* parTokenizer, NodeType parType = XmlElement::TYPE_ELEMENT);
};

#endif

