/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "sections.h"
#include "util.h"

void addSection(block_t** parBlocks, uint32_t parNname, size_t parStart, size_t parLength) {
    block_t* theNewBlock;

    theNewBlock = (block_t*)calloc(1, sizeof(block_t));
    theNewBlock->name = parNname;
    theNewBlock->start = parStart;
    theNewBlock->length = parLength;
    theNewBlock->next = NULL;

    block_t* theParent;
    theParent = *parBlocks;

    if(theParent == NULL) {
        *parBlocks = theNewBlock;
    }
    else {
        while(theParent->next != NULL)
            theParent = theParent->next;

        theParent->next = theNewBlock;
    }
}


void freeSections(block_t** parBlocks)
{
    block_t* theBlocks;
    theBlocks = *parBlocks;

    while(theBlocks && theBlocks->next)
    {
        block_t* tmpBlock = theBlocks->next;
        free(theBlocks);
        theBlocks = tmpBlock;
    }
    free(theBlocks);
    *parBlocks = NULL;
}


block_t* findSections(FILE* fp, int endian) {
    block_t* theBlocks;
    section_head theHead;
    size_t theStart;
    uint8_t done;

    theBlocks = NULL;

    fseek(fp, 0, SEEK_END);
    size_t file_end = ftell(fp);

    fseek(fp, 0, SEEK_SET);
    int first = 1;

    done = 0;
    while(!done) {
        theStart = ftell(fp);

        if (theStart < file_end) {
            fread(&theHead, sizeof(section_head), 1, fp);
            if (endian) {
                theHead.name = swapBytes(theHead.name);
                if (first) {
                    theHead.length = swapBytes(theHead.length);
                    theHead.blocks = swapBytes(theHead.blocks);
                }
            }
            else if (!first) {
                theHead.length = swapBytes(theHead.length);
                theHead.blocks = swapBytes(theHead.blocks);
            }
            first = 0;

            addSection(&theBlocks, theHead.name, theStart, theHead.length);
            fseek(fp, theHead.length - sizeof(theHead), SEEK_CUR);

            if (theHead.name == SECTION_END) {
                done = 1;

                //Find length of data after END block
                theStart = ftell(fp);
                fseek(fp, 0, SEEK_END);
                file_end = ftell(fp);
                addSection(&theBlocks, SECTION_DATA, theStart, (file_end - theStart));
            }
        }
        else {
            done = 1;
        }
    }

    fseek(fp, 0, SEEK_SET);
    return theBlocks;
}


block_t* findSection(block_t* parSections, uint32_t parName) {
    while(parSections) {
        if(parSections->name == parName)
            return parSections;
        parSections = parSections->next;
    }
    return NULL;
}


uint32_t referenceToSection(block_t* parSections, size_t offset) {
    while(parSections) {
        if((offset >= parSections->start) && (offset <= (parSections->start + parSections->length)))
            return parSections->name;
        parSections = parSections->next;
    }
    return -1;
}
