/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef SGB__H
#define SGB__H

#include <stdint.h>


typedef struct {
    float x;
    float z;
    float y;
} point_t;


typedef struct {
    float center_x;
    float center_z;
    float center_y;
    float radius;
} sphere_t;


typedef struct {
    point_t min;
    point_t max;
    uint32_t num;
    uint32_t length;
} flathead_t;

typedef struct {
    sphere_t s1;
    sphere_t s2;
    point_t p1;
    point_t p2;
    uint32_t zero;
    uint32_t id;
} flatdata_t;


typedef struct {
    uint32_t    str1;
    uint32_t    str2;
    point_t     p1;
    point_t     p2;
    point_t     p3;
    point_t     p4;
} occl_t;


typedef struct {
    uint32_t    no;
    uint32_t    str1;
    uint32_t    str2;
    uint32_t    str3;
    uint32_t    num_of_type;
    uint32_t    unknown;
} objhead_t;

typedef struct {
    uint32_t    str1;
    uint32_t    str2;
    uint32_t    str3;
    uint32_t    num_of_type;
    sphere_t    sphere;
    int8_t      matrix_number;
    uint8_t     unknown; //always 0
    uint8_t     matrices;
    uint8_t     subobjects;
} head_t;

typedef struct {
    float   offset_x;
    float   offset_z;
    float   offset_y;

    float   orientation_x;
    float   orientation_z;
    float   orientation_y;

    float   a;
    float   b;
    int32_t parent;
} matrix_t;

typedef struct {
    void* sgb;
    occl_t* occl;
    void* node;
    void* flat;
    void* summ;
    void* end;
    void* data;
} sgb_t;

#endif

