/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "sgb.h"
#include "util.h"
#include "sections.h"
#include "SgbData.hh"
#include <iostream>

void usage()
{
    //std::cout << "sgbconverter version: " << VERSION << std::endl;
    std::cout << "Usage: sgbconverter -in=[sgb|xml] <infile> -out=[sgb|xml] <outfile> [-endian=small|big]" << std::endl;
    std::cout << "Example: sgbconverter -in=sgb alpental.sgb -out=xml alpental.xml" << std::endl;
}


int main(int argc, char* argv[])
{
    char* theInFile     = NULL;
    char* theOutFile    = NULL;

    int theInFormat     = -1;
    int theOutFormat    = -1;
    int theOutEndian    = 1;

    //Parse command line arguments
    if(argc < 5 || argc > 6)
    {
        usage();
        return 0;
    }

    for (int i = 1; i < argc; i++)
    {
        if(strcmp("-in=sgb", argv[i]) == 0)
        {
            theInFile = argv[i+1];
            theInFormat = 0;

        }
        if(strcmp("-in=xml", argv[i]) == 0)
        {
            theInFile = argv[i+1];
            theInFormat = 1;
        }
        if(strcmp("-out=sgb", argv[i]) == 0)
        {
            theOutFile = argv[i+1];
            theOutFormat = 0;
        }
        if(strcmp("-out=xml", argv[i]) == 0)
        {
            theOutFile = argv[i+1];
            theOutFormat= 1;
        }
        
        if(strcmp("-endian=small", argv[i]) == 0)
            theOutEndian = 1;
        
        if(strcmp("-endian=big", argv[i]) == 0)
            theOutEndian = 0;
            
    }

    printf("The in file:  %s\n", theInFile);
    printf("The out file: %s\n", theOutFile);

    if(strcmp(theInFile, theOutFile) == 0)
    {
        std::cout << "Will not read from and save to same file...\n";
        return 1;
    }

    SgbData s;
    if(theInFormat == 0)
        s.readRaw(theInFile);
    else
        s.readXml(theInFile);

    if(theOutFormat == 0)
        s.saveRaw(theOutFile, theOutEndian);
    else
        s.saveXml(theOutFile);
    return 0;
}
