/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "util.h"
#include <string.h>

uint32_t swapBytes(uint32_t parValue) {
    uint32_t theResult = 0;
    uint8_t* tmp = (uint8_t*)(&parValue);

    theResult |= (uint32_t)(*tmp  << 24  & 0xff000000); tmp++;
    theResult |= (uint32_t)((*tmp << 16) & 0x00ff0000); tmp++;
    theResult |= (uint32_t)((*tmp <<  8) & 0x0000ff00); tmp++;
    theResult |= (uint32_t)((*tmp      ) & 0x000000ff);

    return theResult;
}


size_t freadEndian(void *ptr, size_t size, size_t nmemb, FILE *stream, int endian) {
    size_t rval;
    rval = fread(ptr, size, nmemb, stream);
    if(!endian) {
        unsigned int i;
        uint32_t* d = (uint32_t*)ptr;
        for(i = 0; i < (size*nmemb)/sizeof(uint32_t); i++) {
            d[i] = swapBytes(d[i]);
        }
    }
    return rval;
}


char* getString(FILE *fp, size_t pos, char* parData) {
    static char data[300];
    size_t start = ftell(fp);
    fseek(fp, pos-1, SEEK_SET);

    if(!parData)
        parData = data;

    if(fgetc(fp) == 0) {
        fgets(parData, 300, fp);
        fseek(fp, start, SEEK_SET);
        return parData;
    }
    fseek(fp, start, SEEK_SET);
    sprintf(parData, "Not following NULL (%ld)", pos);
    return parData;
}


int isString(uint32_t data) {
    int i;
    char* c = (char*)&data;
    for(i = 0; i < 4; i++) {
        if (!((c[i] >= 'A' && c[i] <= 'Z') || c[i] == ' '))
            return 0;
    }
    return 1;
}


const char* toBin(uint32_t x) {
    int i;
    static char b[33] = {0};
    for(i = 0; i < 33; i++)
        b[i] = 0;

    unsigned int z;
    for (z = 0xffffffFF; z > 0; z >>= 1) {
        strcat(b, ((x & z) == z) ? "1" : "0");
    }

    return b;
}
